@@ -4,6 +4,7 @@
 # existing data is wiped!
 
 use utf8;
+use open ":std", ":encoding(UTF-8)";
 use strict;
 use warnings;
 use DBD::Pg;
@@ -17,11 +18,14 @@ my $dbh = DBI->connect("dbi:Pg:", '', '',
 ) || die "PG connection failed";
 $dbh->do("SET synchronous_commit = off");
 $dbh->do("TRUNCATE unit_prefixes, unit_units");
+$dbh->do("SET client_min_messages = 'error'");
 $dbh->do("ALTER TABLE unit_prefixes ADD COLUMN IF NOT EXISTS ordering serial"); # add temp column to preserve load ordering for dumping
 $dbh->do("ALTER TABLE unit_units ADD COLUMN IF NOT EXISTS ordering serial");
+$dbh->do("RESET client_min_messages");
 
 my $skip_british = 0;
 my @todo;
+my $continued = '';
 
 while (<F>) {
 	# skip over locale specific parts
@@ -29,16 +33,25 @@ while (<F>) {
 	$skip_british = 0 if /^!endvar/;
 	next if ($skip_british);
 
+	if (/\s*(.*)\\$/) {
+		$continued .= $1;
+		next;
+	} elsif ($continued) {
+		s/^\s*//;
+		$_ = "$continued$_";
+		$continued = '';
+	}
+
 	s/#.*//;
 	s/\s+$//;
 	next if /^\s*$/; # skip emtpy lines
 	next if /^!/; # skip pragmas
 	next if /^\+/; # skip units from non-SI systems
-	next if /^[0-9]/; # skip over table contents
-	next if /^ /; # skip over table contents/continued lines
+	#next if /^[0-9]/; # skip over table contents
+	#next if /^\s/; # skip over table contents/continued lines
 	unless (/^(\S+)\s+(.*)/) {
-		print "skipping $_";
-		next;
+		print "unknown line $_\n";
+		exit 1;
 	}
 
 	my ($unit, $def) = ($1, $2);
@@ -92,6 +105,7 @@ do {
 				# something went wrong. It indicates that the new unit could
 				# also be parsed as prefix-otherknownunit, e.g. "ft" vs "f-t"
 				print "Unit $unit has already been used before being defined. Bad.\n";
+				exit 1;
 			}
 			my $ret = $dbh->do("INSERT INTO unit_units (name, unit, shift, definition, dump) VALUES (?, ?, ?, ?, NULL)",
 				undef,
@@ -108,11 +122,14 @@ do {
 } while ($n_todo != $new_n_todo);
 
 print "$new_n_todo units not inserted:\n";
+open my $unres, '>', "definitions.unresolved";
 foreach my $u (@todo) {
 	if ($u->{is_prefix}) {
 		print "Prefix $u->{unit}: $u->{def}\n";
 	} else {
+		print $unres "$u->{unit}\t$u->{def}\n";
 		next if ($u->{error} =~ /dollar|euro|pence|quid|shilling/); # skip currencies so we can see the rest better
 		print "$u->{unit}: $u->{def} ($u->{error})\n";
 	}
 }
+close $unres;
